#include "gtest/gtest.h"
#include "rectangle.h"

using namespace testing;

TEST(RectangleTest, DefaultConstructor) {
    CRectangle rectangle;
    ASSERT_EQ(rectangle.getLength(), 0);
    ASSERT_EQ(rectangle.getWidth(), 0);
}

TEST(RectangleTest, ParameterizedConstructor) {
    CRectangle rectangle(5, 10);
    ASSERT_EQ(rectangle.getLength(), 5);
    ASSERT_EQ(rectangle.getWidth(), 10);
}


TEST(RectangleTest, AreaCalculation) {
    CRectangle rectangle(5, 10);
    ASSERT_EQ(rectangle.calculateArea(), 50);
}

TEST(RectangleTest, AreaCalculationOverloaded) {
    CRectangle rectangle(5, 10);
    ASSERT_EQ(rectangle.calculateArea(5, 10), 50);
}

TEST(RectangleTest, PerimeterCalculation) {
    CRectangle rectangle(5, 10);
    ASSERT_EQ(rectangle.calculatePerimeter(), 30);
}

TEST(RectangleTest, PerimeterCalculationOverloaded) {
    CRectangle rectangle(5, 10);
    ASSERT_EQ(rectangle.calculatePerimeter(5, 10), 30);
}

TEST(RectangleTest, AspectRatioCalculation) {
    CRectangle rectangle(5, 10);
    ASSERT_EQ(rectangle.aspectRatio(), 0.5);
}

TEST(RectangleTest, AspectRatioCalculationOverloaded) {
    CRectangle rectangle(5, 10);
    ASSERT_EQ(rectangle.aspectRatio(5, 10), 0.5);
}

TEST(RectangleTest, Rotation) {
    CRectangle rectangle(5, 10);
    rectangle.rotate();
    ASSERT_EQ(rectangle.getLength(), 10);
    ASSERT_EQ(rectangle.getWidth(), 5);
}

TEST(RectangleTest, RotationOperator) {
    CRectangle rectangle(5, 10);
    !rectangle;
    ASSERT_EQ(rectangle.getLength(), 10);
    ASSERT_EQ(rectangle.getWidth(), 5);
}

TEST(RectangleTest, IncrementOperator) {
    CRectangle rectangle(5, 10);
    rectangle + 2.0;
    ASSERT_EQ(rectangle.getLength(), 7);
    ASSERT_EQ(rectangle.getWidth(), 12);
}

TEST(RectangleTest, MultiplicationOperator) {
    CRectangle rectangle(5, 10);
    rectangle * 2.0;
    ASSERT_EQ(rectangle.getLength(), 10);
    ASSERT_EQ(rectangle.getWidth(), 20);
}

TEST(RectangleTest, ComparisonOperators) {
    CRectangle rectangle1(5, 10);
    CRectangle rectangle2(5, 10);
    CRectangle rectangle3(10, 5);

    ASSERT_TRUE(rectangle1 == rectangle2);
    ASSERT_FALSE(rectangle1 != rectangle2);

    ASSERT_FALSE(rectangle1 == rectangle3);
    ASSERT_TRUE(rectangle1 != rectangle3);
}

TEST(RectangleTest, InputOutputOperators) {
    CRectangle rectangle;
    std::stringstream ss;
    ss << "5 10";
    ss >> rectangle;
    std::stringstream output;
    output << rectangle;
    ASSERT_EQ(output.str(), "Length: 5, Width: 10");
}
